--[[
Author: Starinnia
RealID Toons - Add character information to the BNet alerts your RealID friends generate
$Date: 2010-06-30 19:35:47 +0000 (Wed, 30 Jun 2010) $
$Revision: 25 $
Project Version: r25
contact: codemaster2010 AT gmail DOT com

Copyright (c) 2010 Michael J. Murray aka Lyte of Lothar(US)
All rights reserved unless otherwise explicitly stated.
]]

--save for the future
local myrealm = GetRealmName()

--create a reverse lookup from localized classes to english names
--we need the english names to get the class colors
local reverseClassLookup = {}
for k, v in pairs(LOCALIZED_CLASS_NAMES_FEMALE) do
	reverseClassLookup[v] = k
end
for k, v in pairs(LOCALIZED_CLASS_NAMES_MALE) do
	reverseClassLookup[v] = k
end

local function getFactionString(toonID)
	if not toonID then return "" end --make sure we have a toonID
	if CanCooperateWithToon(toonID) then
		return ""
	else
		local faction = UnitFactionGroup("player") == "Alliance"
		if faction then
			return " - |cffff0000"..strsub(FACTION_HORDE, 1, 1).."|r"
		else
			return " - |cff00aeef"..strsub(FACTION_ALLIANCE, 1, 1).."|r"
		end
	end
end

--pass in the presenceID and classID of the friend
--from this we can obtain the character name, realm and class
--with the classID we can determine their faction
local function constructToonName(presenceID, toonID)
	local _, toon, _, realm, _, _, class = BNGetToonInfo(presenceID)
	
	if RID_TOONS_COLOR then
		--pass the localized class into the lookup table, then the english class to
		local eClass = reverseClassLookup[class]
		local r, g, b
		if _G.CUSTOM_CLASS_COLORS and _G.CUSTOM_CLASS_COLORS[eClass] then
			r, g, b = CUSTOM_CLASS_COLORS[eClass].r*255, CUSTOM_CLASS_COLORS[eClass].g*255, CUSTOM_CLASS_COLORS[eClass].b*255
		elseif RAID_CLASS_COLORS[eClass] then
			r, g, b = RAID_CLASS_COLORS[eClass].r*255, RAID_CLASS_COLORS[eClass].g*255, RAID_CLASS_COLORS[eClass].b*255
		else
			--sometimes BNGetToonInfo does not return class info
			--when this occurs default to a gray color
			r, g, b = 0.63*255, 0.63*255, 0.63*255
		end
		toon = string.format("|cff%02x%02x%02x%s|r", r, g, b, toon)
	end
	
	--make sure realm has a value, sometimes BNGetToonInfo returns an empty string
	if RID_TOONS_REALM and realm ~= myrealm and realm ~= "" then
		toon = toon.." - "..realm
	end
	
	if RID_TOONS_FACTION then
		toon = toon..getFactionString(toonID)
	end
	
	return toon
end

local function filter(self, event, msg, author, ...)
	--the only toast we care about is FRIEND_ONLINE
	if msg ~= "FRIEND_ONLINE" then
		return false, msg, author, ...
	end
	
	local presenceID, givenName, surname, toonID, client, fullName
	local type = strsub(event, 10)
	local info = ChatTypeInfo[type]
	local arg11 = select(9, ...)
	
	--loop through the BNet friends to determine the toon name
	for i = 1, BNGetNumFriends() do
		presenceID, givenName, surname, _, toonID, client = BNGetFriendInfo(i)
		
		--if we can't obtain the data, bail out and let the default alert through
		if not givenName or not surname then
			return false, msg, author, ...
		end
		
		fullName = string.format(BATTLENET_NAME_FORMAT, givenName, surname)
		--double check the client too, this will prevent strangeness when SC2 is released
		if client == BNET_CLIENT_WOW and author == fullName then
			--construct the playerlink and message prefix seperately for easier debugging
			local playerlink = string.format("|HBNplayer:%s:%s:%s:%s:%s|h[%s]|h", author, 7, arg11, Chat_GetChatCategory(type), 0, author)
			local prefix = string.format(BN_INLINE_TOAST_FRIEND_ONLINE, playerlink)
			--add a message to the passed in chat frame that looks like the regular toast
			self:AddMessage(string.format("%s (%s)", prefix, constructToonName(presenceID, toonID)), info.r, info.g, info.b, info.id)
			
			--block the acual toast, it is replaced with our own
			return true, msg, author, ...
			
		--if the client is anything but WoW just return the real name
		elseif author == fullName then
			return false, msg, author, ...
		end
	end
end
ChatFrame_AddMessageEventFilter("CHAT_MSG_BN_INLINE_TOAST_ALERT", filter)

BNToastFrame:HookScript("OnShow", function()
	local topLine = BNToastFrameTopLine
	local bottomLine = BNToastFrameBottomLine
	
	--nil check, depending on what Toasts appear first
	--the bottom line might not have text yet
	if not bottomLine:GetText() then return end
	
	--ensure that this toast is a friend coming online
	if string.match(bottomLine:GetText(), BN_TOAST_ONLINE) then
		local prefix = topLine:GetText()
		local presenceID, givenName, surname, toonID, client, fullName
		
		--if we can't obtain the data, bail out and let the default alert through
		if not givenName or not surname then
			return
		end
		
		--loop through the BNet friends again
		for i = 1, BNGetNumFriends() do
			presenceID, givenName, surname, _, toonID, client = BNGetFriendInfo(i)
			fullName = string.format(BATTLENET_NAME_FORMAT, givenName, surname)
			--if its a WoW client add the toon name, otherwise only display the real name
			if client == BNET_CLIENT_WOW and prefix == fullName then
				topLine:SetFormattedText("%s (%s)", prefix, constructToonName(presenceID, toonID))
				return
			elseif prefix == fullName then
				return
			end
		end
	end
end)
